#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2009-04-17
# Last Modified: $Date: 2009-06-09 15:11:32 +0100 (Tue, 09 Jun 2009) $
# Id:            $Id: Alignment.pm 5570 2009-06-09 14:11:32Z ajb $
# $HeadURL: svn+ssh://intcvs1/repos/svn/new-pipeline-dev/useful_modules/tags/release-10.0/lib/npg_common/Alignment.pm $
#

package npg_common::Alignment;

use strict;
use warnings;
use Carp;
use English qw(-no_match_vars);
use base qw(Class::Accessor);

our $VERSION = do { my ($r) = q$LastChangedRevision: 5570 $ =~ /(\d+)/mxs; $r; };

__PACKAGE__->mk_accessors(qw(bwa_cmd));

sub new {
  my ($class, $ref) = @_;
  $ref ||= {};
  bless $ref, $class;

  return $ref;
}

sub bwa_align_se {
  my ($self, $query_fastq, $sam_out, $ref_root) = @_;

  $query_fastq = $self->_detaint_filename($query_fastq);
  $sam_out = $self->_detaint_filename($sam_out);
  $ref_root = $self->_detaint_filename($ref_root);

  my $bwa_align_cmd = $self->bwa_cmd().qq( aln $ref_root $query_fastq >$sam_out.sai);
  my $bwa_sam_cmd = $self->bwa_cmd().qq( samse $ref_root $sam_out.sai $query_fastq >$sam_out);

  print ">Start doing bwa alignment\n $bwa_align_cmd\n" or croak $ERRNO;

  my $bwa_align_rs = system $bwa_align_cmd;

  if($bwa_align_rs != 0){

    croak "bwa align failed: $bwa_align_cmd";
  }

  my $bwa_sam_rs = system $bwa_sam_cmd;

  if($bwa_sam_rs != 0){

    croak "bwa sam failed: $bwa_sam_cmd";
  }

  print ">Bwa alignment finished\n" or croak $ERRNO;

  return 1;
}

sub _detaint_filename {

  my ($self, $string) = @_;

  my ($dt) = $string =~ /^([\w\.\/\\]+)$/msx; ## no critic (RegularExpressions::ProhibitEscapedMetacharacters)
  if ($dt ne $string) {

    croak "could not detaint $string, got $dt";
  }

  return $dt;
}
1;

__END__

=head1 NAME

npg_common::Alignment

=head1 VERSION

$LastChangedRevision: 5570 $

=head1 SYNOPSIS

  my $oBWAAlignment = npg_common::Alignment->new();

=head1 DESCRIPTION

Class to do bwa alignment base on the fastq file

=head1 SUBROUTINES/METHODS

=head2 new - constructor to create a object

=head2 bwa_align_se -given one fastq file and reference location, doing bwa alignment and output in sam format

=head2 _detaint_filename 

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

Carp
English
Class::Accessor

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Guoying Qi, E<lt>gq1@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2009 GRL, by Guoying Qi

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.8 or,
at your option, any later version of Perl 5 you may have available.

=cut
