#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2009-04-07
# Last Modified: $Date: 2009-06-09 15:11:32 +0100 (Tue, 09 Jun 2009) $
# Id:            $Id: Fastq_Split.pm 5570 2009-06-09 14:11:32Z ajb $
# $HeadURL: svn+ssh://intcvs1/repos/svn/new-pipeline-dev/useful_modules/tags/release-10.0/lib/npg_common/Fastq_Split.pm $
#

package npg_common::Fastq_Split;

use strict;
use warnings;
use Carp;
use English qw(-no_match_vars);
use base qw(Class::Accessor);
use npg_common::Alignment;
use npg_common::Hit_Sequence_SAM;
use File::Basename;
use File::Temp qw(tempfile tempdir);


our $VERSION = do { my ($r) = q$LastChangedRevision: 5570 $ =~ /(\d+)/mxs; $r; };

__PACKAGE__->mk_accessors(qw(bwa_cmd fastqindex_cmd srf_filter_cmd));

sub new {
  my ($class, $ref) = @_;

  $ref ||= {};
  bless $ref, $class;

  return $ref;
}

sub file_split{
  my ($self, $args) = @_;

  my $fastq1 = $args->{fastq1};
  my $fastq2 = $args->{fastq2};
  my $srf_filename = $args->{srf};
  my $srf_filter = $args->{srf_filter};
  my $human_ref = $args->{human_ref};
  my $output_dir = $args->{output_dir};

  if(!$human_ref){

    croak 'Human reference was not given!';
  }

  if(!$output_dir){

    carp 'Output directory was not given!';
  }

  my $work_dir = tempdir(CLEANUP => 0);

  if(!$fastq1 && !$fastq2){

    croak 'At least one fastq file must be provided!';
  }

  print "Trying to split fastq files\n" or croak $ERRNO;

  my ($hit_sequence_sam1, $hit_sequence_sam2);

  if($fastq1){

    $hit_sequence_sam1 = $self->select_human_from_fastq($fastq1, $work_dir.'/1.sam', $human_ref);
  }

  if($fastq2){

    $hit_sequence_sam2 = $self->select_human_from_fastq($fastq2, $work_dir.'/2.sam', $human_ref);
  }

  my ($hit_sequence_ids, $non_hit_sequence_ids);

  print "Spliting fastq files\n" or croak $ERRNO;

  if($hit_sequence_sam1 && $hit_sequence_sam2){

    $hit_sequence_ids = {%{$hit_sequence_sam1->hit_ids()}, %{$hit_sequence_sam2->hit_ids}};
    $non_hit_sequence_ids = $self->merged_non_hit_sequence_ids($hit_sequence_sam1, $hit_sequence_ids);

    undef $hit_sequence_sam1;
    undef $hit_sequence_sam2;

    my $fastq1_human = $output_dir.q{/}.basename($fastq1).q{_human};
    my $fastq1_nonhuman = $output_dir.q{/}.basename($fastq1).q{_nonhuman};
    $self->fastq_split($fastq1, $hit_sequence_ids, $fastq1_human, $fastq1_nonhuman);

    my $fastq2_human = $output_dir.q{/}.basename($fastq2).q{_human};
    my $fastq2_nonhuman = $output_dir.q{/}.basename($fastq2).q{_nonhuman};
    $self->fastq_split($fastq2, $hit_sequence_ids, $fastq2_human, $fastq2_nonhuman);

  }else{

    if($hit_sequence_sam1){

       $hit_sequence_ids = $hit_sequence_sam1->hit_ids();
       $non_hit_sequence_ids = [keys %{$hit_sequence_sam1->non_hit_ids()}];

       undef $hit_sequence_sam1;

       my $fastq1_human = $output_dir.q{/}.basename($fastq1).q{_human};
       my $fastq1_nonhuman = $output_dir.q{/}.basename($fastq1).q{_nonhuman};
       $self->fastq_split($fastq1, $hit_sequence_ids, $fastq1_human, $fastq1_nonhuman);

    }elsif($hit_sequence_sam2){

       $hit_sequence_ids = $hit_sequence_sam2->hit_ids();
       $non_hit_sequence_ids = [keys %{$hit_sequence_sam2->non_hit_ids()}];

       undef $hit_sequence_sam2;

       my $fastq2_human = $output_dir.q{/}.basename($fastq2).q{_human};
       my $fastq2_nonhuman = $output_dir.q{/}.basename($fastq2).q{_nonhuman};
       $self->fastq_split($fastq2, $hit_sequence_ids, $fastq2_human, $fastq2_nonhuman);

    }
  }

  my ($filter_file_hit, $filter_file_non_hit);

  if($srf_filter){

    print "Generating srf filter files in output directory\n" or croak $ERRNO;
    $filter_file_hit = $output_dir.q{/}.q{srf_human.filter};
    $filter_file_non_hit = $output_dir.q{/}.q{srf_nonhuman.filter};

    $self->_srf_filter_file([keys %{$hit_sequence_ids}], $filter_file_hit);
    $self->_srf_filter_file($non_hit_sequence_ids, $filter_file_non_hit);
  }elsif($srf_filename){

    print "Generating srf filter files in temp directory\n" or croak $ERRNO;
    $filter_file_hit = $work_dir.q{/}.basename($srf_filename).q{_human.filter};
    $filter_file_non_hit = $work_dir.q{/}.basename($srf_filename).q{_nonhuman.filter};

    $self->_srf_filter_file([keys %{$hit_sequence_ids}], $filter_file_hit);
    $self->_srf_filter_file($non_hit_sequence_ids, $filter_file_non_hit);
  }

  if($srf_filename){

    $self->split_srf_file($srf_filename, $filter_file_hit, $filter_file_non_hit, $output_dir);
  }

  return 1;
}

sub merged_non_hit_sequence_ids{
  my ($self, $hit_sequence_sam1, $hit_sequence_ids) = @_;

  my $non_hit_sequence_ids;
  my $hit_sequence_ids1 = $hit_sequence_sam1->hit_ids();
  my $non_hit_sequence_ids1 = $hit_sequence_sam1->non_hit_ids();
  foreach my $id ((keys %{$hit_sequence_ids1}, keys %{$non_hit_sequence_ids1})){
    if(!$hit_sequence_ids->{$id}){
      push @{$non_hit_sequence_ids}, $id;
    }
  }

  return $non_hit_sequence_ids;
}
sub split_srf_file{
  my ($self, $srf_filename, $filter_file_hit, $filter_file_non_hit, $output_dir) = @_;

  print "Spliting srf file\n" or croak $ERRNO;
  my $srf_human = $output_dir.q{/}.basename($srf_filename).q{_human};
  my $srf_non_human = $output_dir.q{/}.basename($srf_filename).q{_nonhuman};

  $self->srf_filter($srf_filename, $filter_file_hit, $srf_human );
  $self->srf_filter($srf_filename, $filter_file_non_hit, $srf_non_human);

  return 1;
}

sub select_human_from_fastq{
  my ($self, $fastq, $sam_out, $human_ref) = @_;

  my $alignment = npg_common::Alignment->new({
                                bwa_cmd =>$self->bwa_cmd(),
                                });

  $alignment->bwa_align_se($fastq, $sam_out, $human_ref);

  my $hit_sequence_sam = npg_common::Hit_Sequence_SAM->new({filename =>$sam_out});
  $hit_sequence_sam->parsing_file();

  return $hit_sequence_sam;
}

sub fastq_split {
  my ($self, $fastq_fn, $select_ids, $select_fn, $rest_fn) = @_;

  $fastq_fn = $self->_detaint_filename($fastq_fn);
  $select_fn = $self->_detaint_filename($select_fn);
  $rest_fn = $self->_detaint_filename($rest_fn);

  open my $fastq_fh, q[<], $fastq_fn or croak "$fastq_fn:\n$ERRNO"; ## no critic (InputOutput::RequireBriefOpen)  
  open my $select_fh, q[>], $select_fn or croak "$select_fn:\n$ERRNO"; ## no critic (InputOutput::RequireBriefOpen)
  open my $rest_fh, q[>], $rest_fn or croak "$rest_fn:\n$ERRNO"; ## no critic (InputOutput::RequireBriefOpen)

  while(my $id = <$fastq_fh>) {

    chomp $id;
    ($id) = $id =~ /^@([A-Za-z0-9_.:-]+)\/?\d?/mxs;

    my $seq = <$fastq_fh>;
    my $seq_name  = <$fastq_fh>;
    my $qual = <$fastq_fh>;

    if($select_ids->{$id}){

      print {$select_fh} "$id\n$seq$seq_name$qual" or croak $ERRNO;
    }else{

      print {$rest_fh} "$id\n$seq$seq_name$qual"or croak $ERRNO;
    }

  }

  close $fastq_fh or croak $ERRNO;
  close $select_fh or croak $ERRNO;
  close $rest_fh or croak $ERRNO;

  return 1;
}

sub fastq_select_index {
  my ($self, $fastq_index_fn, $select_ids, $select_fn) = @_;

  $select_fn = $self->_detaint_filename($select_fn);

  my $fetch_cmd = $self->fastqindex_cmd().qq{ --index=$fastq_index_fn --fetch};

  #Here put ids in the command line, if too many, they can't put in a file
  #sequence ids in fastq could end with /1 or /2 but ids in sam file without them
  foreach my $id (keys %{$select_ids}){
    $fetch_cmd .= " $id";
  };

  $fetch_cmd .= " > $select_fn";

  my $fetch_cmd_rs = system $fetch_cmd;

  if($fetch_cmd_rs != 0){

    croak "failed to fetch sequence\n";
  }

  return 1;
}

sub srf_filter{
  my ($self, $srf_fn, $filter_fn, $out_fn) = @_;

  $srf_fn = $self->_detaint_filename($srf_fn);
  $filter_fn= $self->_detaint_filename($filter_fn);
  $out_fn = $self->_detaint_filename($out_fn);

  my $srf_filter_cmd = $self->srf_filter_cmd().qq{ -f file=$filter_fn $srf_fn $out_fn};
  print ">$srf_filter_cmd\n" or croak "$ERRNO";

  my $srf_filter_cmd_rs = system $srf_filter_cmd;

  if($srf_filter_cmd_rs != 0){

    croak "srf_filter failed for $srf_fn\n";
  }

  return $srf_filter_cmd_rs;
}

sub _srf_filter_file{
  my ($self, $ids_hash, $output_fn) = @_;

  $output_fn = $self->_detaint_filename($output_fn);
  open my $output_fh, q[>], $output_fn or croak "$output_fn:\n$ERRNO";

  foreach my $id (@{$ids_hash}){
    print {$output_fh} "read=$id\n" or croak 'cannot print';
  }

  close $output_fh or  croak $ERRNO;

  return 1;
}

sub _get_seq_ids_from_file {
  my ($self, $id_fn) = @_;

  my %seq_ids = ();

  open my $id_fh, q[<], $id_fn or croak "$id_fn:\n$ERRNO";

  while(my $id = <$id_fh>) {

    chomp $id;
    $seq_ids{$id}++;
  }
  close $id_fh or croak $ERRNO;

  return \%seq_ids;
}

sub _fastq_indexing{
  my ($self, $fastq_fn, $fastq_index_fn) = @_;

  $fastq_fn = $self->_detaint_filename($fastq_fn);

  my $fastqindex_cmd = $self->fastqindex_cmd().qq{ --build --index=$fastq_index_fn $fastq_fn};
  print ">$fastqindex_cmd\n" or croak "$ERRNO";

  my $fastqindex_cmd_rs = system $fastqindex_cmd;

  if($fastqindex_cmd_rs != 0){

    croak "fastqindex failed for $fastq_fn\n";
  }

  return  $fastqindex_cmd_rs;
}

sub _detaint_filename {

  my ($self, $string) = @_;

  my ($dt) = $string =~ /^([\w\.\/\\]+)$/msx; ## no critic (RegularExpressions::ProhibitEscapedMetacharacters)

  if ($dt ne $string) {

    croak "could not detaint $string, got $dt";
  }

  return $dt;
}

1;

__END__

=head1 NAME

npg_common::Fastq_Split

=head1 VERSION

$LastChangedRevision: 5570 $

=head1 SYNOPSIS

  my $oFastqSplit = npg_common::Fastq_Split->new();

=head1 DESCRIPTION

Class to split fastq files or srf file given a list of ids

=head1 SUBROUTINES/METHODS

=head2 new - constructor to create a object

=head2 select_human_from_fastq - given a fastq file, do bwa alignment with human reference and split the ids into two sets(human and nonhuman)

=head2 file_split - given one or two fastq files, or srf file, split them into human and nonhuman files

=head2 fastq_split - given a fastq file and a list ids in hash, and two output file names, split the files into two

=head2 fastq_select_index - given a list of ids in hash, and fastq index file, generate a new fastq file based on the id list

=head2 srf_filter - given a list of ids in a file, call srf_filter command to generate a new srf file based on the id list

=head2 split_srf_file - given a srf file, two sets of ids and output_dir, using srf_filter command to split the srf file into two

=head2 srf_filter_file - given a list of ids in hash, generate srf_filter file

=head2 _get_seq_ids_from_file - read the id list out from a text file

=head2 _fastq_indexing - given a fastq file and output index file name, call fastqindex command to generate the index fastq file

=head2 _detaint_filename - check file name

=head2 merged_non_hit_sequence_ids - merged hit sequence ids together and put the others into the non-hit list

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

Carp
English
Class::Accessor

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Guoying Qi, E<lt>gq1@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2009 GRL, by Guoying Qi

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.8 or,
at your option, any later version of Perl 5 you may have available.

=cut
