#########
# Author:        Marina Gourtovaia
# Maintainer:    $Author: ajb $
# Created:       05 May 2009
# Last Modified: $Date: 2009-06-09 15:11:32 +0100 (Tue, 09 Jun 2009) $
# Id:            $Id: visio_histo_google.pm 5570 2009-06-09 14:11:32Z ajb $
# $HeadURL: svn+ssh://intcvs1/repos/svn/new-pipeline-dev/useful_modules/tags/release-10.0/lib/npg_common/diagram/visio_histo_google.pm $
#

package npg_common::diagram::visio_histo_google;

use strict;
use warnings;

use Readonly;
use Carp;
use English qw(-no_match_vars);

## no critic(ProhibitParensWithBuiltins Capitalization ProhibitMixedCaseSubs ProhibitManyArgs)

our $VERSION   = do { my ($r) = q$LastChangedRevision: 5570 $ =~ /(\d+)/mxs; $r; };


Readonly::Scalar our $DELIMETER   => q[&];
Readonly::Scalar our $GOOGLE_URL  => q[http://chart.apis.google.com/chart?];

sub new {
    my $class = shift;
    my $self = {};
    bless $self, $class;

    $self->{cht_chart_type}         = ['cht' , 'bvg'];
    $self->{chco_chart_colour}      = ['chco', '4D89F9'];
    $self->{chtt_chart_title}       = ['chtt', 'batch+X,run+Y,+lane+Z'];
    $self->{chs_chart_size}         = ['chs' , '300x250'];
    $self->{chd_chart_data}         = ['chd' , 't:20,30,40,23'];
    $self->{chds_chart_min_max}     = ['chds', '0,40'];
    $self->{chxt_chart_axises}      = ['chxt', 'x,y'];
    $self->{chxr_chart_axis_labels} = ['chxr', '0,1,4|1,0,40,10'];

    return $self;
}


sub get_diagram_string {

    my $self = shift;
    my @params = ();
    foreach my $key (sort keys %{$self}) {
        my $pair = $self->{$key};
        push @params, $pair->[0] . q[=] . $pair->[1];
    }
    my $url = $GOOGLE_URL . join("$DELIMETER", @params);
    return $url;
}


sub set_data {

    my ($self, $data) = @_;
    $self->{chd_chart_data} = ['chd' , 't:' . join(q[,], @{$data})];
    return 1;
}


sub set_axisY_min_max {

    my ($self, $min, $max) = @_;
    $self->{chds_chart_min_max} = ['chds', $min . q[,] . $max];
    return 1;
}


sub set_chart_title {

    my ($self, $title) = @_;
    $self->{chtt_chart_title} = ['chtt', join(q[+], split(q[ ],  $title))];
    return 1;
}


sub set_chart_labels {

    my ($self, $xmin, $xmax, $xdelta, $ymin, $ymax, $ydelta) = @_;
    my @x = (0, $xmin, $xmax, $xdelta);
    my @y = (1, $ymin, $ymax, $ydelta);
    my $labels = join(q[,], @x) . q[|] . join(q[,], @y);
    $self->{chxr_chart_axis_labels} = ['chxr', $labels];
    return 1;
}


sub set_chart_size {

    my ($self, $width, $height) = @_;
    $self->{chs_chart_size} = ['chs' , $width . 'x' . $height];
    return 1;
}




1;
__END__

=head1 NAME

npg_common::diagram::visio_histo_google - an API for producing bar diagrams with Google Chart API

=head1 VERSION

$LastChangedRevision: 5570 $

=head1 SYNOPSIS

  use npg_common::diagram::visio_histo_google;

=head1 DESCRIPTION

npg_common::diagram::visio_histo_google

=head1 USAGE

  my $dia = npg_common::diagram::visio_histo_google->new();
  my $array = [1,2,2,3,5,5,7];
  $dia->set_title(q[My Chart]);
  $dia->set_data($array);
  $dia->set_axisY_min_max(0,7);
  my $url = $dia->get_diagram_string();

=head1 OPTIONS

=head1 SUBROUTINES/METHODS

=head2 new returns a npg_common::diagram::visio_histo_google object
  $dia = npg_common::diagram::visio_histo_google->new();

=head2 get_diagram_string returns a URL for generating a Google chart

=head2 set_chart_title sets the title of the chart

=head2 set_data  sets data

=head2 set_axisY_min_max sets axis Y min and max values

=head2 set_chart_labels

=head2 set_chart_size  

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item strict

=item warnings

=item Carp

=item English -no_match_vars

=item Readonly

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=cut

