#########
# Author:        Marina Gourtovaia
# Maintainer:    $Author: mg8 $
# Created:       03 April 2009
# Last Modified: $Date: 2009-04-08 18:30:40 +0100 (Wed, 08 Apr 2009) $
# Id:            $Id: reference.pm 4830 2009-04-08 17:30:40Z mg8 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/trunk/lib/npg_common/extractor/reference.pm $
#

package npg_common::extractor::reference;

use strict;
use warnings;
use Carp;
use English qw(-no_match_vars);
use Exporter;

our $VERSION   = do { my ($r) = q$LastChangedRevision: 5570 $ =~ /(\d+)/mxs; $r; };

## no critic (ProhibitExplicitISA)
our @ISA       = qw(Exporter);
our @EXPORT_OK = qw(get_chromosome);

# Perl trim function to remove whitespace from the start and end of the string
sub _trim {
    my $string = shift;
    $string =~ s/^\s+//smx;
    $string =~ s/\s+$//smx;
    return $string;
}


sub get_chromosome {

    my ($ref_path, $extract_path, $tag) = @_;

    ## no critic (ProhibitMagicNumbers)
    if (@_ != 3) {
        croak q[Wrong number of arguments in ] . __PACKAGE__ . q[::get_chromosome];
    }
    ## use critic

    my $tag_flag = 0;
    my $comment_char = q[>];
    my $start_tag = $comment_char . $tag;

    ## no critic (RequireBriefOpen)
    open my $ffull, q[<], $ref_path or croak qq[Cannot open $ref_path];
    open my $fex, q[>], $extract_path or croak qq[Cannot create $extract_path];

    while (my $line = <$ffull>) {

        $line = _trim($line);
        if ($tag_flag) {
            if ($line =~ /^$comment_char/smx) {
	        last;
            }
            print {$fex} qq[$line\n] or croak $ERRNO;
        } else {
            if ($line eq $start_tag) {
                $tag_flag = 1;
                print {$fex} qq[$line\n] or croak $ERRNO;
            }
        }
    }

    close $ffull or croak $ERRNO;
    close $fex or croak $ERRNO;

    if (!$tag_flag) {
        croak qq[Chromosome with tag $tag not found];
    }

    return 1;
}

1;

__END__

=head1 NAME

npg_common::extractor::reference

=head1 VERSION

$LastChangedRevision: 5570 $

=head1 SYNOPSIS
  This module is for extracting parts of rest references.

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 get_chromosome - extracts data for one chromosome
  my $tag = q[<20];
  get_chromosome($ref_path, $extract_path, $tag); where
  $ref_path is a path to a reference in text format;
  $extract_path - a path to a file that should be produced;
  $tag - a tag that identifies the required chromosome.

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item strict

=item warnings

=item Carp

=item English -no_match_vars

=item Readonly

=item Exporter

=item Math::Round qw(round);

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: mg8 $

=head1 LICENSE AND COPYRIGHT

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=cut


