use strict;
use warnings;
use Test::More tests => 10;
use English qw(-no_match_vars);
use Carp;


my $source1 = q[t/data/s1.fastq];
my $source2 = q[t/data/s2.fastq];

my @subs = qw(line_count get_equally_spaced_reads);
use_ok( 'npg_common::extractor::fastq', @subs);

{
    can_ok(__PACKAGE__, 'line_count');
    can_ok(__PACKAGE__, 'get_equally_spaced_reads');
}


{
   my $count = line_count($source1);
   is($count, '104', 'line count');

}

{

   # 26 reads in test file
   
   my $dest1   = q[t/data/s1e.fastq];
   my $dest2   = q[t/data/s2e.fastq];

   
   my $num_reads_required = 27;
   eval {
       get_equally_spaced_reads($source1, $source2, $dest1, $dest2, $num_reads_required);
   };
   like($EVAL_ERROR, qr/is\ too\ short/smix, 'croak if the input file is too short');

   eval {
       get_equally_spaced_reads($source1, $source2, $dest1, $dest2);
   };
   like($EVAL_ERROR, qr/Wrong\ number\ of/smix, 'croak if a wrong number of arguments to get_short_fastq()');
   
   $num_reads_required = 3;
   get_equally_spaced_reads($source1, $source2, $dest1, $dest2, $num_reads_required);

   ok(-e $dest1, 'output for read1 exists');
   ok(-e $dest2, 'output for read2 exists');

   my $test = q[t/data/test.fastq];
   my $similar = 0;
   my $exe = qq[diff $test $dest1];
   open my $fh, q[-|], $exe or croak $ERRNO;
   if(!<$fh>) {
      $similar = 1;
   }
   close $fh;
   
   ok($similar, 'output as expected for the first file');

   $similar = 0;
   $exe = qq[diff $test $dest2];
   open $fh, q[-|], $exe or croak $ERRNO;
   if(!<$fh>) {
      $similar = 1;
   }
   close $fh;
   ok($similar, 'output as expected for the second file');

   unlink $dest1;
   unlink $dest2;
}

