use strict;
use warnings;
use Test::More tests => 7;

use English qw(-no_match_vars);
use Carp;

 my @subs = qw(get_chromosome);
 use_ok( 'npg_common::extractor::reference', @subs);

{
    can_ok(__PACKAGE__, 'get_chromosome');
}


{
    eval {
       get_chromosome(q[source], q[30]);
    };
    like($EVAL_ERROR, qr/Wrong\ number\ of\ arguments/smix, 'croak on wrong number of arguments to get_chromosome');
}


{
    my $source = q[t/data/simple_reference];
    my $target = q[t/data/extracted.txt];

    get_chromosome($source, $target, q[20]);

    ok ( -e $target, 'output file created');

    open my $fh, q[<], $target or croak $ERRNO; 
    local $RS = undef;
    my $contents = <$fh>;
    close $fh or croak $ERRNO;
    unlink $target;

    is($contents, qq[>20\naaaaaaaaaaaaa\nbbbbbbbbbbbbb\n], 'chromosome extracted');
  
    eval {
       get_chromosome($source, $target, q[30]);
    };
    like($EVAL_ERROR, qr/Chromosome\ with\ tag/smix, 'croak on tag not found');

    eval {
      get_chromosome(q[dummy], $target, q[30]);
    };
    like($EVAL_ERROR, qr/Cannot\ open\ dummy/smix, 'croak on source not open');
}


