#########
# Author:        ajb
# Maintainer:    $Author: ajb $
# Created:       2008-08-07
# Last Modified: $Date: 2009-06-09 15:11:32 +0100 (Tue, 09 Jun 2009) $
# Id:            $Id: 10-image_graph.t 5570 2009-06-09 14:11:32Z ajb $
# $HeadURL: svn+ssh://intcvs1/repos/svn/new-pipeline-dev/useful_modules/tags/release-10.0/t/10-image_graph.t $
#

use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use SangerPaths qw{core ensembl badger trackingnpg tracking};
use Test::More tests => 22;
use Digest::MD5;

my $md5_object = Digest::MD5->new();

use_ok('npg_common::image::graph');

{
  my $graph = npg_common::image::graph->new({});
  isa_ok($graph, 'npg_common::image::graph', '$graph');
  my $args = {
    'width'        => 400,
    'height'       => 200,
    'title'        => 'test data',
    'x_labels_vertical' => 1,
    'x_label'      => 'Run ID',
    'y_label'      => 'PF Yield (GBases)',

  };
  my $data = [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]];
  my $png;

  eval { $png = $graph->plotter($data, $args, q{lines}, 1); };
  is($EVAL_ERROR, q{}, 'no croak on generation of png graph image - with array rotate');
  $md5_object->add($png);
  is($md5_object->hexdigest(), q{fa1ed7a609b3a419033c847f3e8923f5}, 'md5 hex digest of png is correct');

  eval { $png = $graph->plotter($data, $args, q{lines}, undef); };
  is($EVAL_ERROR, q{}, 'no croak on generation of png graph image - without array rotate');
  $md5_object->add($png);
  is($md5_object->hexdigest(), q{b91a7cacfcef7c3468557ee9f1fe3498}, 'md5 hex digest of png is correct');
}
{
  my $graph = npg_common::image::graph->new({});
  isa_ok($graph, 'npg_common::image::graph', '$graph');
  my $args = {
    'title'        => 'test data',
    'x_labels_vertical' => 1,
    'x_label'      => 'Run ID',
    'y_label'      => 'PF Yield (GBases)',
    'legend'       => ['test_1','test_2'],

  };
  my $data = [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9],[1,2,3,4,5,6,7,8,9,10]];
  my $png;

  eval { $png = $graph->plotter($data, $args, q{lines}, 1); };
  is($EVAL_ERROR, q{}, 'no croak on generation of png graph image - with legend');
  $md5_object->add($png);
  is($md5_object->hexdigest(), q{3dc81fa8ad18148517bcfbf3a369e7f0}, 'md5 hex digest of png is correct');
}
{
  my $graph = npg_common::image::graph->new({});
  isa_ok($graph, 'npg_common::image::graph', '$graph');
  my $args = {
    'title'        => 'test data',
    'x_labels_vertical' => 1,
    'x_label'      => 'Run ID',
    'y_label'      => 'PF Yield (GBases)',
    'legend'       => ['test_1','test_2'],

  };
  my $data = [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9],[1,2,3,4,5,6,7,8,9,10]];
  my $png;

  eval { $png = $graph->plotter($data); };
  is($EVAL_ERROR, q{}, 'no croak on generation of png graph image - just data');
  $md5_object->add($png);
  is($md5_object->hexdigest(), q{12c92a0f825b014cb13790fba345d186}, 'md5 hex digest of png is correct');

  eval { $png = $graph->plotter($data, undef, undef, 1); };
  is($EVAL_ERROR, q{}, 'no croak on generation of png graph image - just data and rotate');
  $md5_object->add($png);
  is($md5_object->hexdigest(), q{0e08dcc8d89cf5e4d5b9b23938e5a421}, 'md5 hex digest of png is correct');
}
{
  my $graph = npg_common::image::graph->new({});
  isa_ok($graph, 'npg_common::image::graph', '$graph');
  my $args = {
    'title'        => 'test data',
    'x_labels_vertical' => 1,
    'x_label'      => 'Run ID',
    'y_label'      => 'PF Yield (GBases)',
    'legend'       => ['test_1','test_2'],

  };
  my $data = [];
  my $png;

  eval { $png = $graph->plotter($data, undef, undef, 1); };
  is($EVAL_ERROR, q{}, 'no croak on generation of png graph image - no scalar@{$data}');
  $md5_object->add($png);
  is($md5_object->hexdigest(), q{14798143a22d510c6ef786a6450ee405}, 'md5 hex digest of png is correct');
}
{
  my $graph = npg_common::image::graph->new({});
  isa_ok($graph, 'npg_common::image::graph', '$graph');
  $graph->colours([qw(red white)]);
  my $args = {
    'x_label'      => 'Instrument',
    'y_label'      => 'Days',
    'y_max_value'  => 50,
    'y_min_value'  => 0,

  };
  my $data = [[qw(IL1 IL2 IL3 IL4 IL5)],[10,20,30,40,50]];
  my $png;

  eval { $png = $graph->plotter($data, $args, q{bars}, 1); };
  is($EVAL_ERROR, q{}, 'no croak on generation of png graph image');
  $md5_object->add($png);
  is($md5_object->hexdigest(), q{f178977b90823fb0f59362191990f185}, 'md5 hex digest of png is correct');

  my $png2;
  $graph->colours([qw(white red)]);
  $data = [[qw(IL1 IL2 IL3 IL4 IL5)],[5,10,15,20,25]];
  eval { $png2 = $graph->plotter($data, $args, q{bars}, 1); };
  is($EVAL_ERROR, q{}, 'no croak on generation of png graph image');
  $md5_object->add($png2);
  is($md5_object->hexdigest(), q{ff6a950d2cb963d615f67cc1673a60db}, 'md5 hex digest of png is correct');
}
