#########
# Author:        ajb
# Maintainer:    $Author: ajb $
# Created:       2008-08-07
# Last Modified: $Date: 2009-06-09 15:11:32 +0100 (Tue, 09 Jun 2009) $
# Id:            $Id: 10-image_merge.t 5570 2009-06-09 14:11:32Z ajb $
# $HeadURL: svn+ssh://intcvs1/repos/svn/new-pipeline-dev/useful_modules/tags/release-10.0/t/10-image_merge.t $
#

use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use SangerPaths qw{core ensembl badger trackingnpg tracking};
use Test::More tests => 18;
use Digest::MD5;
use npg_common::image::graph;

my $md5_object = Digest::MD5->new();

use_ok('npg_common::image::merge');

{
  my $merge = npg_common::image::merge->new({});
  isa_ok($merge, 'npg_common::image::merge', '$merge');
  my $arg_refs = {
     format => 'spamcodjfijdijcvdocv',
 };
  eval { $merge->merge_images($arg_refs); };
  like($EVAL_ERROR, qr{spamcodjfijdijcvdocv\ is\ not\ a\ supported\ format\ of\ merged\ images}, 'croaked on running $merge->merge_images() as unsupported format');
}
{
  my $merge = npg_common::image::merge->new({});
  my $arg_refs = {
     format => 'table_portrait',
 };
  eval { $merge->merge_images($arg_refs); };
  like($EVAL_ERROR, qr{no\ data\ provided\ for\ drawing\ images}, 'croaked on running $merge->merge_images() as no data provided');
}
{
  my $merge = npg_common::image::merge->new({});
  my $arg_refs = {
     format => 'table_portrait',
     data   => [],
 };
  eval { $merge->merge_images($arg_refs); };
  like($EVAL_ERROR, qr{no\ data\ provided\ for\ drawing\ images}, 'croaked on running $merge->merge_images() as no column data provided');
}
{
  my $merge = npg_common::image::merge->new({});
  my $arg_refs = {
    format => 'table_portrait',
    rows   => '8',
    cols   => '3',
    data   => [
    [
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9],[521,522,923,524,55,524,513,512,515,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]]
    ],
    [
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]]
    ],
    [
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9],[521,522,923,524,55,524,513,512,515,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]]
    ],
    [
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]]
    ],
    [
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9],[521,522,923,524,55,524,513,512,515,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]]
    ],
    [
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]]
    ],
    [
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9],[521,522,923,524,55,524,513,512,515,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]]
    ],
    [
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]]
    ]
    ],
    column_headings => ['Lane', 'All', 'Call', '% Base Calls'],
    y_min_values => [0,0,0],
    y_max_values => [1500,1500,undef],
    
    row_headings => [qw(1 2 3 4 5 6 7 8)],
    args_for_image => {
      'width'        => 126,
      'height'       => 84,
      'x_labels_vertical' => 1,
    },
  };
  my $png;
  eval { $png = $merge->merge_images($arg_refs); };
  is($EVAL_ERROR, q{}, 'no croak on running $merge->merge_images()');
  $md5_object->add($png);
  is($md5_object->hexdigest(), q{2bf6133eb97ae612c2ca9a2fc8856ca6}, 'md5 hex digest of png is correct');
}
{
  my $merge = npg_common::image::merge->new({});
  my $arg_refs = {
    format => 'table_landscape',
    rows   => '3',
    cols   => '8',
    data   => [
    [
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9],[521,522,923,524,55,524,513,512,515,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9],[521,522,923,524,55,524,513,512,515,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9],[521,522,923,524,55,524,513,512,515,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]]
    ],
    [
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9],[521,522,923,524,55,524,513,512,515,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9],[521,522,923,524,55,524,513,512,515,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]]
    ],
    [
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9],[521,522,923,524,55,524,513,512,515,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9],[521,522,923,524,55,524,513,512,515,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9],[521,522,923,524,55,524,513,512,515,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]]
    ],
    ],
    row_headings => ['All', 'Call', '% Base Calls'],
    y_min_values => [0,0,0],
    y_max_values => [1500,1500,undef],
    
    column_headings => [qw(Lane 1 2 3 4 5 6 7 8)],
    args_for_image => {
      'width'        => 126,
      'height'       => 84,
      'x_labels_vertical' => 1,
    },
  };
  my $png;
  eval { $png = $merge->merge_images($arg_refs); };
  is($EVAL_ERROR, q{}, 'no croak on running $merge->merge_images()');
  $md5_object->add($png);
  is($md5_object->hexdigest(), q{7feb32172ae5b7d9d86cc655f80f328d}, 'md5 hex digest of png is correct');
}
{
  my $graph = npg_common::image::graph->new({});
  isa_ok($graph, 'npg_common::image::graph', '$graph');
  $graph->colours([qw(red white)]);
  my $args = {
    'x_label'      => 'Instrument',
    'y_label'      => 'Days',
    'y_max_value'  => 50,
    'y_min_value'  => 0,

  };
  my $data = [[qw(IL1 IL2 IL3 IL4 IL5)],[10,20,30,40,50]];
  my $png;

  eval { $png = $graph->plotter($data, $args, q{bars}, 1); };
  my $png2;
  $graph->colours([qw(white red)]);
  $data = [[qw(IL1 IL2 IL3 IL4 IL5)],[5,10,15,20,25]];
  eval { $png2 = $graph->plotter($data, $args, q{bars}, 1); };

  my $merge = npg_common::image::merge->new({});

  my $arg_refs = {
    format => 'overlay_all_images_exactly',
    images => [$png, $png2],
  };
  my $result_png;
  eval { $result_png = $merge->merge_images($arg_refs); };
  is($EVAL_ERROR, q{}, 'no croak on running $merge->merge_images()');
  $md5_object->add($result_png);
  is($md5_object->hexdigest(), q{0050d9ff38d06b335a6c2bddf58034d1}, 'md5 hex digest of result_png is correct');
  
}

{
  my $merge = npg_common::image::merge->new({});

  my $arg_refs = {
    format => 'gantt_chart_vertical',
    'x_label'      => 'Instrument',
    'y_label'      => 'Days',
    'y_max_value'  => 50,
    'y_min_value'  => 0,
    'x_axis'       => [qw(IL2 IL3 IL4 IL5 IL6)],
    'data_points'  => [[50,20,30,40,50],[5,10,15,20,25],[0,0,10,18,0],[0,0,5,0,0]],
    'height'       => 400,
    'width'        => 800,
    'y_tick_number' => 5,
  };
  my $result_png;
  eval { $result_png = $merge->merge_images($arg_refs); };
  is($EVAL_ERROR, q{}, 'no croak on running $merge->merge_images()');
  $md5_object->add($result_png);
  is($md5_object->hexdigest(), q{b8935ec19ac305602276806e1409ec93}, 'md5 hex digest of result_png is correct');

}
{
  my $merge = npg_common::image::merge->new({});

  my $arg_refs = {
    format => 'gantt_chart_vertical',
    'x_label'      => 'Instrument',
    'y_label'      => 'Days',
    'y_max_value'  => 50,
    'y_min_value'  => 0,
    'x_axis'       => [qw(IL2 IL3 IL4 IL5 IL6)],
    'data_points'  => [[50,20,30,40,50],[5,10,15,20,25],[0,0,10,18,0],[0,0,5,0,0]],
    'add_points'   => [[45,15,25,0,0],[undef,undef,15,0,undef],[undef,undef,10,0,1]],
    'height'       => 400,
    'width'        => 800,
    'y_tick_number' => 5,
  };
  my $result_png;
  eval { $result_png = $merge->merge_images($arg_refs); };
  is($EVAL_ERROR, q{}, 'no croak on running $merge->merge_images()');
  $md5_object->add($result_png);
  is($md5_object->hexdigest(), q{dd24ffe504dd4b6404003826c09d22e5}, 'md5 hex digest of result_png is correct');

}
{
  my $merge = npg_common::image::merge->new({});

  my $arg_refs = {
    format => 'gantt_chart_vertical',
    'x_label'      => 'Instrument',
    'y_label'      => 'Days',
    'y_max_value'  => 50,
    'y_min_value'  => 0,
    'x_axis'       => [qw(IL2 IL3 IL4 IL5 IL6)],
    'data_points'  => [[50,20,30,40,50],[5,10,15,20,25],[0,0,10,18,0],[0,0,5,0,0]],
    'add_points'   => [[45,15,25,0,0],[undef,undef,15,0,undef],[undef,undef,10,0,1]],
    'height'       => 400,
    'width'        => 800,
    'colour_of_block' => q{black},
    'y_tick_number' => 5,
  };
  my $result_png;
  eval { $result_png = $merge->merge_images($arg_refs); };
  is($EVAL_ERROR, q{}, 'no croak on running $merge->merge_images()');
  $md5_object->add($result_png);
  is($md5_object->hexdigest(), q{3a838fd2d98396353efcb9b65ff14551}, 'md5 hex digest of result_png is correct');

}
__END__  
    [
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]]
    ],
    [
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9],[521,522,923,524,55,524,513,512,515,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]]
    ],
    [
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]]
    ],
    [
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9],[521,522,923,524,55,524,513,512,515,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]]
    ],
    [
    [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]],[[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]]
    ]
